package com.company.test

import com.company.test.sys.CustomInitServiceListener
import com.vaadin.flow.component.page.AppShellConfigurator
import com.vaadin.flow.component.page.Push
import com.vaadin.flow.server.PWA
import com.vaadin.flow.theme.Theme
import io.jmix.core.CoreProperties
import io.jmix.core.JmixModules
import io.jmix.core.Resources
import io.jmix.flowui.exception.UiExceptionHandlers
import io.jmix.flowui.sys.JmixServiceInitListener
import io.jmix.flowui.view.ViewRegistry
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.SpringApplication
import org.springframework.boot.autoconfigure.SpringBootApplication
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties
import org.springframework.boot.context.event.ApplicationStartedEvent
import org.springframework.boot.context.properties.ConfigurationProperties
import org.springframework.boot.context.properties.ConfigurationPropertiesScan
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Primary
import org.springframework.context.event.EventListener
import org.springframework.core.env.Environment
import org.springframework.scheduling.annotation.EnableScheduling
import javax.sql.DataSource

@Push
@Theme(value = "jmix-test")
@PWA(name = "Jmix Test", shortName = "Jmix Test")
@EnableScheduling
@SpringBootApplication
@ConfigurationPropertiesScan
open class JmixTestApplication : AppShellConfigurator {

    companion object {
        @JvmStatic
        fun main(args: Array<String>) {
            SpringApplication.run(JmixTestApplication::class.java, *args)
        }
    }

    @Autowired
    private val environment: Environment? = null

    @Bean
    @Primary
    @ConfigurationProperties("main.datasource")
    open fun dataSourceProperties(): DataSourceProperties = DataSourceProperties()

    @Bean
    @Primary
    @ConfigurationProperties("main.datasource.hikari")
    open fun dataSource(dataSourceProperties: DataSourceProperties): DataSource =
        dataSourceProperties.initializeDataSourceBuilder().build()

    @EventListener
    open fun printApplicationUrl(event: ApplicationStartedEvent?) {
        LoggerFactory.getLogger(JmixTestApplication::class.java).info(
            "Application started at http://localhost:"
                    + (environment?.getProperty("local.server.port") ?: "")
                    + (environment?.getProperty("server.servlet.context-path") ?: "")
        )
    }

    @Primary
    @Bean
    open fun extDepartmentService(
        viewRegistry: ViewRegistry,
        uiExceptionHandlers: UiExceptionHandlers,
        coreProperties: CoreProperties,
        modules: JmixModules,
        resources: Resources
    ): JmixServiceInitListener {
        return CustomInitServiceListener(viewRegistry, uiExceptionHandlers, coreProperties, modules, resources)
    }
}
